import { jsx as _jsx, Fragment as _Fragment } from "react/jsx-runtime";
import { EditorView, keymap } from "@codemirror/view";
import { ReplayDataRecorder } from "@liqvid/recording";
import { bind } from "@liqvid/utils/misc";
import { icon } from "./icon";
// the actual thingy that gets exported
export class CodeRecorder extends ReplayDataRecorder {
    constructor() {
        super();
        bind(this, ["extension"]);
    }
    extension(keys = []) {
        // record document changes
        const updateListener = EditorView.updateListener.of(update => {
            if (!this.manager || this.manager.paused || !this.manager.active)
                return;
            // get selection change (if any)
            const transactions = update.transactions
                .map(t => {
                if (!t.selection)
                    return null;
                const range = t.selection.ranges[0];
                return [range.anchor, range.head];
            })
                .filter(Boolean)
                .slice(-1);
            this.capture(this.manager.getTime(), [
                update.changes.toJSON(),
                ...transactions
            ]);
        });
        // record special key presses
        const keyListener = keymap.of(keys.map(key => ({
            key,
            run: () => {
                if (!this.manager || this.manager.paused || !this.manager.active)
                    return false;
                this.capture(this.manager.getTime(), key);
                return false;
            }
        })));
        return [updateListener, keyListener];
    }
}
/* helper functions */
// function lookupKeyForEditor(cm, name, handle) {
//   for (let i = 0; i < cm.state.keyMaps.length; i++) {
//     const result = CodeMirror.lookupKey(name, cm.state.keyMaps[i], handle, cm);
//     if (result) return result;
//   }
//   return (cm.options.extraKeys && CodeMirror.lookupKey(name, cm.options.extraKeys, handle, cm))
//     || CodeMirror.lookupKey(name, cm.options.keyMap, handle, cm)
// }
function KeySaveComponent(props) {
    return (_jsx(_Fragment, { children: _jsx("textarea", { readOnly: true, value: JSON.stringify(props.data) }, void 0) }, void 0));
}
export const CodeRecording = {
    enabled: () => true,
    icon,
    key: "codemirror",
    name: "Code",
    recorder: new CodeRecorder(),
    saveComponent: KeySaveComponent,
    title: "Record code"
};
